@@ -1,19 +1,25 @@
-# Copyright (c) 2008-2014 Simplistix Ltd
-# See license.txt for license details.
-
 import atexit
 import os
 import warnings
+from pathlib import Path
 
 from re import compile
 from tempfile import mkdtemp
+from typing import Union, Sequence, TYPE_CHECKING, Tuple, Callable
+
 from testfixtures.comparison import compare
-from testfixtures.compat import basestring
 from testfixtures.utils import wrap
 
 from .rmtree import rmtree
 
 
+PathStrings = Union[str, Tuple[str]]
+
+
+if TYPE_CHECKING:
+    import py.path
+
+
 class TempDirectory:
     """
     A class representing a temporary directory on disk.
@@ -26,8 +32,8 @@ class TempDirectory:
     :param create: If `True`, the temporary directory will be created
                    as part of class instantiation.
 
-    :param path: If passed, this should be a string containing a
-                 physical path to use as the temporary directory. When
+    :param path: If passed, this should be a string containing an
+                 absolute path to use as the temporary directory. When
                  passed, :class:`TempDirectory` will not create a new
                  directory to use.
 
@@ -39,17 +45,24 @@ class TempDirectory:
     instances = set()
     atexit_setup = False
 
-    #: The physical path of the :class:`TempDirectory` on disk
+    #: The absolute path of the :class:`TempDirectory` on disk
     path = None
 
-    def __init__(self, ignore=(), create=True, path=None, encoding=None):
+    def __init__(
+            self,
+            path: Union[str, Path, 'py.path.local'] = None,
+            *,
+            ignore: Sequence[str] = (),
+            create: bool = None,
+            encoding: str = None
+    ):
         self.ignore = []
         for regex in ignore:
             self.ignore.append(compile(regex))
-        self.path = path
+        self.path = str(path) if path else None
         self.encoding = encoding
         self.dont_remove = bool(path)
-        if create:
+        if create or (path is None and create is None):
             self.create()
 
     @classmethod
@@ -60,7 +73,7 @@ class TempDirectory:
                 '%s' % ('\n'.join(i.path for i in cls.instances))
                 )
 
-    def create(self):
+    def create(self) -> 'TempDirectory':
         """
         Create a temporary directory for this instance to use if one
         has not already been created.
@@ -74,7 +87,7 @@ class TempDirectory:
             self.__class__.atexit_setup = True
         return self
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """
         Delete the temporary directory and anything in it.
         This :class:`TempDirectory` cannot be used again unless
@@ -87,7 +100,7 @@ class TempDirectory:
             self.instances.remove(self)
 
     @classmethod
-    def cleanup_all(cls):
+    def cleanup_all(cls) -> None:
         """
         Delete all temporary directories associated with all
         :class:`TempDirectory` objects.
@@ -95,8 +108,13 @@ class TempDirectory:
         for i in tuple(cls.instances):
             i.cleanup()
 
-    def actual(self,
-               path=None, recursive=False, files_only=False, followlinks=False):
+    def actual(
+            self,
+            path: PathStrings = None,
+            recursive: bool = False,
+            files_only: bool = False,
+            followlinks: bool = False,
+    ):
         path = self._join(path) if path else self.path
 
         result = []
@@ -130,7 +148,7 @@ class TempDirectory:
             filtered.append(path)
         return filtered
 
-    def listdir(self, path=None, recursive=False):
+    def listdir(self, path: PathStrings = None, recursive: bool = False):
         """
         Print the contents of the specified directory.
 
@@ -159,8 +177,14 @@ class TempDirectory:
         for n in actual:
             print(n)
 
-    def compare(self, expected, path=None, files_only=False, recursive=True,
-                followlinks=False):
+    def compare(
+            self,
+            expected: Sequence[str],
+            path: PathStrings = None,
+            files_only: bool = False,
+            recursive: bool = True,
+            followlinks: bool = False,
+    ):
         """
         Compare the expected contents with the actual contents of the temporary
         directory. An :class:`AssertionError` will be raised if they are not the
@@ -185,96 +209,26 @@ class TempDirectory:
         :param files_only: If specified, directories will be excluded from
                            the list of actual paths used in the comparison.
 
-        :param recursive: If passed as ``False``, only the direct contents of
+        :param recursive: If ``False``, only the direct contents of
                           the directory specified by ``path`` will be included
                           in the actual contents used for comparison.
 
-        :param followlinks: If passed as ``True``, symlinks and hard links
+        :param followlinks: If ``True``, symlinks and hard links
                             will be followed when recursively building up
                             the actual list of directory contents.
         """
+
+        __tracebackhide__ = True
+    
         compare(expected=sorted(expected),
                 actual=tuple(self.actual(
                     path, recursive, files_only, followlinks
                 )),
                 recursive=False)
 
-    def check(self, *expected):
-        """
-        .. deprecated:: 4.3.0
-
-        Compare the contents of the temporary directory with the
-        expected contents supplied.
-
-        This method only checks the root of the temporary directory.
-
-        :param expected: A sequence of strings containing the names
-                         expected in the directory.
-        """
-        compare(expected, tuple(self.actual()), recursive=False)
-
-    def check_dir(self, dir, *expected):
-        """
-        .. deprecated:: 4.3.0
-
-        Compare the contents of the specified subdirectory of the
-        temporary directory with the expected contents supplied.
-
-        This method will only check the contents of the subdirectory
-        specified and will not recursively check subdirectories.
-
-        :param dir: The subdirectory to check, which can be:
-
-                     * A tuple of strings, indicating that the
-                       elements of the tuple should be used as directory
-                       names to traverse from the root of the
-                       temporary directory to find the directory to be
-                       checked.
-
-                     * A forward-slash separated string, indicating
-                       the directory or subdirectory that should be
-                       traversed to from the temporary directory and
-                       checked.
-
-        :param expected: A sequence of strings containing the names
-                         expected in the directory.
-        """
-        compare(expected, tuple(self.actual(dir)), recursive=False)
-
-    def check_all(self, dir, *expected):
-        """
-        .. deprecated:: 4.3.0
-
-        Recursively compare the contents of the specified directory
-        with the expected contents supplied.
-
-        :param dir: The directory to check, which can be:
-
-                     * A tuple of strings, indicating that the
-                       elements of the tuple should be used as directory
-                       names to traverse from the root of the
-                       temporary directory to find the directory to be
-                       checked.
-
-                     * A forward-slash separated string, indicating
-                       the directory or subdirectory that should be
-                       traversed to from the temporary directory and
-                       checked.
-
-                     * An empty string, indicating that the whole
-                       temporary directory should be checked.
-
-        :param expected: A sequence of strings containing the paths
-                         expected in the directory. These paths should
-                         be forward-slash separated and relative to
-                         the root of the temporary directory.
-        """
-        compare(expected, tuple(self.actual(dir, recursive=True)),
-                recursive=False)
-
     def _join(self, name):
         # make things platform independent
-        if isinstance(name, basestring):
+        if isinstance(name, str):
             name = name.split('/')
         relative = os.sep.join(name).rstrip(os.sep)
         if relative.startswith(os.sep):
@@ -285,7 +239,7 @@ class TempDirectory:
                 )
         return os.path.join(self.path, relative)
 
-    def makedir(self, dirpath):
+    def makedir(self, dirpath: PathStrings):
         """
         Make an empty directory at the specified path within the
         temporary directory. Any intermediate subdirectories that do
@@ -297,13 +251,13 @@ class TempDirectory:
 
                         * A forward-slash separated string.
 
-        :returns: The full path of the created directory.
+        :returns: The absolute path of the created directory.
         """
         thepath = self._join(dirpath)
         os.makedirs(thepath)
         return thepath
 
-    def write(self, filepath, data, encoding=None):
+    def write(self, filepath: PathStrings, data: Union[bytes, str], encoding: str = None):
         """
         Write the supplied data to a file at the specified path within
         the temporary directory. Any subdirectories specified that do
@@ -319,14 +273,17 @@ class TempDirectory:
 
                          * A forward-slash separated string.
 
-        :param data: A string containing the data to be written.
+        :param data:
+
+          :class:`bytes` containing the data to be written, or a :class:`str`
+          if ``encoding`` has been supplied.
 
         :param encoding: The encoding to be used if data is not bytes. Should
                          not be passed if data is already bytes.
 
-        :returns: The full path of the file written.
+        :returns: The absolute path of the file written.
         """
-        if isinstance(filepath, basestring):
+        if isinstance(filepath, str):
             filepath = filepath.split('/')
         if len(filepath) > 1:
             dirpath = self._join(filepath[:-1])
@@ -340,7 +297,7 @@ class TempDirectory:
             f.write(data)
         return thepath
 
-    def getpath(self, path):
+    def as_string(self, path: Union[str, Sequence[str]] = None) -> str:
         """
         Return the full path on disk that corresponds to the path
         relative to the temporary directory that is passed in.
@@ -351,11 +308,44 @@ class TempDirectory:
 
                      * A forward-slash separated string.
 
-        :returns: A string containing the full path.
-        """
-        return self._join(path)
+        :returns: A string containing the absolute path.
 
-    def read(self, filepath, encoding=None):
+        """
+        return self.path if path is None else self._join(path)
+
+    #: .. deprecated:: 7
+    #:
+    #:   Use :meth:`as_string` instead.
+    getpath = as_string
+
+    def as_path(self, path: PathStrings = None) -> Path:
+        """
+        Return the :class:`~pathlib.Path` that corresponds to the path
+        relative to the temporary directory that is passed in.
+
+        :param path: The path to the file to create, which can be:
+
+                     * A tuple of strings.
+
+                     * A forward-slash separated string.
+        """
+        return Path(self.path if path is None else self._join(path))
+
+    def as_local(self, path: PathStrings = None) -> 'py.path.local':
+        """
+        Return the :class:`py.path.local` that corresponds to the path
+        relative to the temporary directory that is passed in.
+
+        :param path: The path to the file to create, which can be:
+
+                     * A tuple of strings.
+
+                     * A forward-slash separated string.
+        """
+        from py.path import local
+        return local(self.path if path is None else self._join(path))
+
+    def read(self, filepath: PathStrings, encoding: str = None) -> Union[bytes, str]:
         """
         Reads the file at the specified path within the temporary
         directory.
@@ -372,7 +362,10 @@ class TempDirectory:
 
         :param encoding: The encoding used to decode the data in the file.
 
-        :returns: A string containing the data read.
+        :returns:
+
+          The contents of the file as a :class:`str` or :class:`bytes`, if ``encoding``
+          is not specified.
         """
         with open(self._join(filepath), 'rb') as f:
             data = f.read()
@@ -388,7 +381,7 @@ class TempDirectory:
         self.cleanup()
 
 
-def tempdir(*args, **kw):
+def tempdir(*args, **kw) -> Callable[[Callable], Callable]:
     """
     A decorator for making a :class:`TempDirectory` available for the
     duration of a test function.
