@@ -1,5 +1,4 @@
-from __future__ import print_function
-from logging import getLogger, ERROR
+from logging import getLogger, ERROR, Filter
 from textwrap import dedent
 from unittest import TestCase
 from warnings import catch_warnings
@@ -15,6 +14,11 @@ two = getLogger('two')
 child = getLogger('one.child')
 
 
+class DummyFilter(Filter):
+    def filter(self, _):
+        return True
+
+
 class TestLogCapture(TestCase):
 
     def test_simple(self):
@@ -135,6 +139,10 @@ class TestLogCapture(TestCase):
         root = getLogger()
         child = getLogger('child')
 
+        # Add a dummy filter so we can verify it is swapped out
+        # during the capture, and swapped back in after `uninstall`.
+        root.addFilter(DummyFilter())
+
         # Lets also record the handlers for these loggers before
         # we start the test:
 
@@ -146,6 +154,11 @@ class TestLogCapture(TestCase):
         old_root_level=root.level
         old_child_level=child.level
 
+        # Also record the filters:
+
+        old_root_filters = root.filters[:]
+        old_child_filters = child.filters[:]
+
         # Now the test:
 
         try:
@@ -159,6 +172,9 @@ class TestLogCapture(TestCase):
             assert root.level == 1
             assert child.level == 1
 
+            assert root.filters == []
+            assert child.filters == []
+
             child.info('2')
             assert str(l1) == (
                 "root INFO\n  1\n"
@@ -167,6 +183,11 @@ class TestLogCapture(TestCase):
             assert str(l2) == (
                 "child INFO\n  2"
             )
+
+            # Add a dummy filter to the child,
+            # which should be removed by `uninstall`.
+            child.addFilter(DummyFilter())
+
             l2.uninstall()
             l1.uninstall()
             assert root.level == 49
@@ -180,6 +201,10 @@ class TestLogCapture(TestCase):
         assert root.handlers == before_root
         assert child.handlers == before_child
 
+        # Also check the filters were restored:
+        assert root.filters == old_root_filters
+        assert child.filters == old_child_filters
+
     def test_uninstall_all(self):
         before_handlers_root = root.handlers[:]
         before_handlers_child = child.handlers[:]
@@ -408,16 +433,16 @@ class TestCheckPresent(object):
             root.error('j2')
         with ShouldAssert(dedent("""\
                 ignored:
-                [('root', 'ERROR', 'j1'), ('root', 'ERROR', 'j2')]
+                [('root', 'ERROR', 'j1'), ('root', 'ERROR', 'three'), ('root', 'ERROR', 'j2')]
                 
                 same:
-                [('root', 'INFO', 'one')]
+                [('root', 'INFO', 'one'), ('root', 'WARNING', 'two')]
                 
                 expected:
-                [('root', 'WARNING', 'two'), ('root', 'ERROR', 'three')]
+                [('root', 'ERROR', 'three')]
                 
                 actual:
-                [('root', 'ERROR', 'three'), ('root', 'WARNING', 'two')]""")):
+                []""")):
             log.check_present(
                 ('root', 'INFO', 'one'),
                 ('root', 'WARNING', 'two'),
@@ -482,11 +507,6 @@ class TestCheckPresent(object):
                 []""")):
             log.check_present('two')
 
-    def test_bad_params(self):
-        # not needed if we didn't have to support Python 2!
-        with ShouldAssert('order_matters is the only keyword parameter'):
-            LogCapture(install=False).check_present(foo='bar')
-
     def test_multiple_identical_expected_order_matters(self):
         with LogCapture() as log:
             root.info('one')
